@@ -49,12 +49,19 @@
 /* name of the duplicate delivery database */
 #define FNAME_DELIVERDB "/deliver.db"
 
+typedef struct duplicate_key {
+    const char *id;
+    const char *to;
+    const char *date;
+} duplicate_key_t;
+
+#define DUPLICATE_INITIALIZER { NULL, NULL, NULL }
+
 int duplicate_init(const char *fname, int myflags);
 
-time_t duplicate_check(char *id, int idlen, const char *to, int tolen);
-void duplicate_log(char *msgid, const char *name, char *action);
-void duplicate_mark(char *id, int idlen, const char *to, int tolen, time_t mark,
-		    unsigned long uid);
+time_t duplicate_check(duplicate_key_t *dkey);
+void duplicate_log(duplicate_key_t *dkey, char *action);
+void duplicate_mark(duplicate_key_t *dkey, time_t mark, unsigned long uid);
 int duplicate_find(char *msgid, int (*proc)(), void *rock);
 
 int duplicate_prune(int seconds, struct hash_table *expire_table);
