@@ -700,33 +700,37 @@ static int user_sub(const char *userid, const char *mboxname)
 static int copy_local(struct mailbox *mailbox, unsigned long uid)
 {
     uint32_t recno;
-    struct index_record record;
-    char *oldfname, *newfname;
+    struct index_record oldrecord;
     int r;
 
     for (recno = 1; recno <= mailbox->i.num_records; recno++) {
-	r = mailbox_read_index_record(mailbox, recno, &record);
+	r = mailbox_read_index_record(mailbox, recno, &oldrecord);
 	if (r) return r;
-	if (record.uid == uid) {
-	    /* store the old record, expunged */
-	    record.system_flags |= FLAG_EXPUNGED;
-	    r = mailbox_rewrite_index_record(mailbox, &record);
-	    if (r) return r;
 
-	    /* create the new record */
-	    record.system_flags &= ~FLAG_EXPUNGED;
-	    record.uid = mailbox->i.last_uid + 1;
+	/* found the record, renumber it */
+	if (oldrecord.uid == uid) {
+	    char *oldfname, *newfname;
+	    struct index_record newrecord;
+
+	    /* create the new record as a clone of the old record */
+	    newrecord = oldrecord;
+	    newrecord.uid = mailbox->i.last_uid + 1;
 
 	    /* copy the file in to place */
-	    oldfname = xstrdup(mailbox_message_fname(mailbox, uid));
-	    newfname = xstrdup(mailbox_message_fname(mailbox, record.uid));
+	    oldfname = xstrdup(mailbox_message_fname(mailbox, oldrecord.uid));
+	    newfname = xstrdup(mailbox_message_fname(mailbox, newrecord.uid));
 	    r = mailbox_copyfile(oldfname, newfname, 0);
 	    free(oldfname);
 	    free(newfname);
 	    if (r) return r;
 
-	    /* and append the new record (a clone apart from the EXPUNGED flag) */
-	    r = mailbox_append_index_record(mailbox, &record);
+	    /* append the new record */
+	    r = mailbox_append_index_record(mailbox, &newrecord);
+	    if (r) return r;
+
+	    /* and expunge the old record */
+	    oldrecord.system_flags |= FLAG_EXPUNGED;
+	    r = mailbox_rewrite_index_record(mailbox, &oldrecord);
 
 	    /* done - return */
 	    return r;
@@ -1258,7 +1262,7 @@ static int mailbox_full_update(const char *mboxname)
 	r2 = sync_parse_response("EXPUNGE", sync_in, NULL);
 	if (r2) {
 	    syslog(LOG_ERR, "SYNCERROR: failed to expunge in cleanup %s",
-		   mailbox->name);
+		   mboxname);
 	}
     }
 
@@ -1424,7 +1428,7 @@ static int do_seen(char *user, char *uniqueid)
 {
     int r = 0;
     struct seen *seendb = NULL;
-    struct seendata sd;
+    struct seendata sd = SEENDATA_INITIALIZER;
 
     if (verbose) 
         printf("SEEN %s %s\n", user, uniqueid);
@@ -1591,7 +1595,10 @@ int do_folders(struct sync_name_list *mboxname_list,
 
     r = reserve_messages(mboxname_list, master_folders,
 			 replica_folders, reserve_guids);
-    if (r) goto bail;
+    if (r) {
+	syslog(LOG_ERR, "reserve messages: failed: %s", error_message(r));
+	goto bail;
+    }
 
     /* Tag folders on server which still exist on the client. Anything
      * on the server which remains untagged can be deleted immediately */
@@ -1610,7 +1617,11 @@ int do_folders(struct sync_name_list *mboxname_list,
     for (rfolder = replica_folders->head; rfolder; rfolder = rfolder->next) {
 	if (rfolder->mark) continue;
 	r = folder_delete(rfolder->name);
-	if (r) goto bail;
+	if (r) {
+	    syslog(LOG_ERR, "folder_delete(): failed: %s '%s'", 
+		   rfolder->name, error_message(r));
+	    goto bail;
+	}
     }
 
     /* Need to rename folders in an order which avoids dependancy conflicts
@@ -1625,8 +1636,11 @@ int do_folders(struct sync_name_list *mboxname_list,
 	for (item = rename_folders->head; item; item = item->next) {
 	    if (item->done) continue;
 
-	    item2 = sync_rename_lookup(rename_folders, item->newname);
-	    if (item2 && !item2->done) continue;
+	    /* don't skip rename to different partition */
+	    if (strcmp(item->oldname, item->newname)) {
+		item2 = sync_rename_lookup(rename_folders, item->newname);
+		if (item2 && !item2->done) continue;
+	    }
 
 	    /* Found unprocessed item which should rename cleanly */
 	    r = folder_rename(item->oldname, item->newname, item->part);
@@ -2604,6 +2618,7 @@ void replica_connect(const char *channel)
 		int r;
 		int optval = 1;
 		socklen_t optlen = sizeof(optval);
+		struct protoent *proto = getprotobyname("TCP");
 
 		r = setsockopt(sync_backend->sock, SOL_SOCKET, SO_KEEPALIVE, &optval, optlen);
 		if (r < 0) {
@@ -2612,7 +2627,7 @@ void replica_connect(const char *channel)
 #ifdef TCP_KEEPCNT
 		optval = config_getint(IMAPOPT_TCP_KEEPALIVE_CNT);
 		if (optval) {
-		    r = setsockopt(sync_backend->sock, SOL_TCP, TCP_KEEPCNT, &optval, optlen);
+		    r = setsockopt(sync_backend->sock, proto->p_proto, TCP_KEEPCNT, &optval, optlen);
 		    if (r < 0) {
 			syslog(LOG_ERR, "unable to setsocketopt(TCP_KEEPCNT): %m");
 		    }
@@ -2621,7 +2636,7 @@ void replica_connect(const char *channel)
 #ifdef TCP_KEEPIDLE
 		optval = config_getint(IMAPOPT_TCP_KEEPALIVE_IDLE);
 		if (optval) {
-		    r = setsockopt(sync_backend->sock, SOL_TCP, TCP_KEEPIDLE, &optval, optlen);
+		    r = setsockopt(sync_backend->sock, proto->p_proto, TCP_KEEPIDLE, &optval, optlen);
 		    if (r < 0) {
 			syslog(LOG_ERR, "unable to setsocketopt(TCP_KEEPIDLE): %m");
 		    }
@@ -2630,7 +2645,7 @@ void replica_connect(const char *channel)
 #ifdef TCP_KEEPINTVL
 		optval = config_getint(IMAPOPT_TCP_KEEPALIVE_INTVL);
 		if (optval) {
-		    r = setsockopt(sync_backend->sock, SOL_TCP, TCP_KEEPINTVL, &optval, optlen);
+		    r = setsockopt(sync_backend->sock, proto->p_proto, TCP_KEEPINTVL, &optval, optlen);
 		    if (r < 0) {
 			syslog(LOG_ERR, "unable to setsocketopt(TCP_KEEPINTVL): %m");
 		    }
@@ -2661,6 +2676,11 @@ void replica_connect(const char *channel)
     /* links to sockets */
     sync_in = sync_backend->in;
     sync_out = sync_backend->out;
+
+    /* Force use of LITERAL+ so we don't need two way communications */
+    prot_setisclient(sync_in, 1);
+    prot_setisclient(sync_out, 1);
+
 }
 
 static void replica_disconnect()
@@ -2694,8 +2714,8 @@ void do_daemon(const char *sync_log_file, const char *sync_shutdown_file,
 /* ====================================================================== */
 
 static struct sasl_callback mysasl_cb[] = {
-    { SASL_CB_GETOPT, &mysasl_config, NULL },
-    { SASL_CB_CANON_USER, &mysasl_canon_user, NULL },
+    { SASL_CB_GETOPT, (mysasl_cb_ft *) &mysasl_config, NULL },
+    { SASL_CB_CANON_USER, (mysasl_cb_ft *) &mysasl_canon_user, NULL },
     { SASL_CB_LIST_END, NULL, NULL }
 };
 
